Package: qusage
Version: 2.46.0
Date: 2013-01-20
Title: qusage: Quantitative Set Analysis for Gene Expression
Authors@R: c(person("Christopher Bolen", "Developer", role = c("aut",
        "cre"), email = "cbolen1@gmail.com"), person("Gur Yaari",
        "Developer", role = "aut"), person("Juilee Thakar",
        "Developer", role = "aut"), person("Hailong Meng", 
        "Developer", role = "aut"), person("Jacob Turner", 
        "Developer", role = "aut"), person("Derek Blankenship",
        "Developer", role = "aut"), person("Steven Kleinstein",
        "Developer", role = "aut"))
Author: Christopher Bolen and Gur Yaari, with contributions from Juilee
        Thakar, Hailong Meng, Jacob Turner, Derek Blankenship, and
        Steven Kleinstein
Maintainer: Christopher Bolen <cbolen1@gmail.com>
Depends: R (>= 2.10), limma (>= 3.14), methods
Imports: utils, Biobase, nlme, emmeans, fftw
Description: This package is an implementation the Quantitative Set
        Analysis for Gene Expression (QuSAGE) method described in
        (Yaari G. et al, Nucl Acids Res, 2013). This is a novel Gene
        Set Enrichment-type test, which is designed to provide a
        faster, more accurate, and easier to understand test for gene
        expression studies. qusage accounts for inter-gene correlations
        using the Variance Inflation Factor technique proposed by Wu et
        al. (Nucleic Acids Res, 2012). In addition, rather than simply
        evaluating the deviation from a null hypothesis with a single
        number (a P value), qusage quantifies gene set activity with a
        complete probability density function (PDF). From this PDF, P
        values and confidence intervals can be easily extracted.
        Preserving the PDF also allows for post-hoc analysis (e.g.,
        pair-wise comparisons of gene set activity) while maintaining
        statistical traceability. Finally, while qusage is compatible
        with individual gene statistics from existing methods (e.g.,
        LIMMA), a Welch-based method is implemented that is shown to
        improve specificity. The QuSAGE package also includes a mixed
        effects model implementation, as described in (Turner JA et al, 
        BMC Bioinformatics, 2015), and a meta-analysis framework as 
        described in (Meng H, et al. PLoS Comput Biol. 2019).
        For questions, contact Chris Bolen (cbolen1@gmail.com) or 
        Steven Kleinstein (steven.kleinstein@yale.edu)
License: GPL (>= 2)
URL: http://clip.med.yale.edu/qusage
biocViews: GeneSetEnrichment, Microarray, RNASeq, Software,
        ImmunoOncology
git_url: https://git.bioconductor.org/packages/qusage
git_branch: RELEASE_3_23
git_last_commit: e78b0eb
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:07:23 UTC; biocbuild
